@@ -12,7 +12,7 @@ import math
 import numpy as np
 
 from matplotlib import (
-    artist, cbook, colors as mcolors, lines, text as mtext, path as mpath)
+    artist, colors as mcolors, lines, text as mtext, path as mpath)
 from matplotlib.collections import (
     LineCollection, PolyCollection, PatchCollection, PathCollection)
 from matplotlib.colors import Normalize
@@ -28,12 +28,6 @@ def _norm_angle(a):
     return a
 
 
-@cbook.deprecated("3.1")
-def norm_angle(a):
-    """Return the given angle normalized to -180 < *a* <= 180 degrees."""
-    return _norm_angle(a)
-
-
 def _norm_text_angle(a):
     """Return the given angle normalized to -90 < *a* <= 90 degrees."""
     a = (a + 180) % 180
@@ -42,12 +36,6 @@ def _norm_text_angle(a):
     return a
 
 
-@cbook.deprecated("3.1")
-def norm_text_angle(a):
-    """Return the given angle normalized to -90 < *a* <= 90 degrees."""
-    return _norm_text_angle(a)
-
-
 def get_dir_vector(zdir):
     """
     Return a direction vector.
@@ -152,13 +140,7 @@ class Line3D(lines.Line2D):
     def set_3d_properties(self, zs=0, zdir='z'):
         xs = self.get_xdata()
         ys = self.get_ydata()
-
-        try:
-            # If *zs* is a list or array, then this will fail and
-            # just proceed to juggle_axes().
-            zs = np.full_like(xs, fill_value=float(zs))
-        except TypeError:
-            pass
+        zs = np.broadcast_to(zs, xs.shape)
         self._verts3d = juggle_axes(xs, ys, zs, zdir)
         self.stale = True
 
@@ -168,16 +150,16 @@ class Line3D(lines.Line2D):
 
         Parameters
         ----------
-        x : array_like
-            The x-data to be plotted
-        y : array_like
-            The y-data to be plotted
-        z : array_like
-            The z-data to be plotted
+        x : array-like
+            The x-data to be plotted.
+        y : array-like
+            The y-data to be plotted.
+        z : array-like
+            The z-data to be plotted.
 
         Notes
         -----
-        Accepts x, y, z arguments or a single array_like (x, y, z)
+        Accepts x, y, z arguments or a single array-like (x, y, z)
         """
         if len(args) == 1:
             self._verts3d = args[0]
@@ -191,8 +173,8 @@ class Line3D(lines.Line2D):
 
         Returns
         -------
-        verts3d : length-3 tuple or array_likes
-            The current data as a tuple or array_likes
+        verts3d : length-3 tuple or array-like
+            The current data as a tuple or array-like.
         """
         return self._verts3d
 
@@ -222,12 +204,6 @@ def _path_to_3d_segment(path, zs=0, zdir='z'):
     return seg3d
 
 
-@cbook.deprecated("3.1")
-def path_to_3d_segment(path, zs=0, zdir='z'):
-    """Convert a path to a 3D segment."""
-    return _path_to_3d_segment(path, zs=zs, zdir=zdir)
-
-
 def _paths_to_3d_segments(paths, zs=0, zdir='z'):
     """Convert paths from a collection object to 3D segments."""
 
@@ -237,12 +213,6 @@ def _paths_to_3d_segments(paths, zs=0, zdir='z'):
     return segs
 
 
-@cbook.deprecated("3.1")
-def paths_to_3d_segments(paths, zs=0, zdir='z'):
-    """Convert paths from a collection object to 3D segments."""
-    return _paths_to_3d_segments(paths, zs=zs, zdir=zdir)
-
-
 def _path_to_3d_segment_with_codes(path, zs=0, zdir='z'):
     """Convert a path to a 3D segment with path codes."""
 
@@ -258,12 +228,6 @@ def _path_to_3d_segment_with_codes(path, zs=0, zdir='z'):
     return seg3d, list(codes)
 
 
-@cbook.deprecated("3.1")
-def path_to_3d_segment_with_codes(path, zs=0, zdir='z'):
-    """Convert a path to a 3D segment with path codes."""
-    return _path_to_3d_segment_with_codes(path, zs=zs, zdir=zdir)
-
-
 def _paths_to_3d_segments_with_codes(paths, zs=0, zdir='z'):
     """
     Convert paths from a collection object to 3D segments with path codes.
@@ -279,14 +243,6 @@ def _paths_to_3d_segments_with_codes(paths, zs=0, zdir='z'):
     return list(segments), list(codes)
 
 
-@cbook.deprecated("3.1")
-def paths_to_3d_segments_with_codes(paths, zs=0, zdir='z'):
-    """
-    Convert paths from a collection object to 3D segments with path codes.
-    """
-    return _paths_to_3d_segments_with_codes(paths, zs=zs, zdir=zdir)
-
-
 class Line3DCollection(LineCollection):
     """
     A collection of 3D lines.
@@ -301,7 +257,7 @@ class Line3DCollection(LineCollection):
         """
         Set 3D segments.
         """
-        self._segments3d = np.asanyarray(segments)
+        self._segments3d = segments
         LineCollection.set_segments(self, [])
 
     def do_3d_projection(self, renderer):
@@ -399,12 +355,6 @@ def _get_patch_verts(patch):
         return []
 
 
-@cbook.deprecated("3.1")
-def get_patch_verts(patch):
-    """Return a list of vertices for the path of a patch."""
-    return _get_patch_verts(patch)
-
-
 def patch_2d_to_3d(patch, z=0, zdir='z'):
     """Convert a Patch to a Patch3D object."""
     verts = _get_patch_verts(patch)
@@ -531,6 +481,8 @@ class Path3DCollection(PathCollection):
         self._offsets3d = juggle_axes(xs, ys, np.atleast_1d(zs), zdir)
         self._facecolor3d = self.get_facecolor()
         self._edgecolor3d = self.get_edgecolor()
+        self._sizes3d = self.get_sizes()
+        self._linewidth3d = self.get_linewidth()
         self.stale = True
 
     def do_3d_projection(self, renderer):
@@ -539,14 +491,39 @@ class Path3DCollection(PathCollection):
 
         fcs = (_zalpha(self._facecolor3d, vzs) if self._depthshade else
                self._facecolor3d)
-        fcs = mcolors.to_rgba_array(fcs, self._alpha)
-        self.set_facecolors(fcs)
-
         ecs = (_zalpha(self._edgecolor3d, vzs) if self._depthshade else
                self._edgecolor3d)
+        sizes = self._sizes3d
+        lws = self._linewidth3d
+
+        # Sort the points based on z coordinates
+        # Performance optimization: Create a sorted index array and reorder
+        # points and point properties according to the index array
+        z_markers_idx = np.argsort(vzs)[::-1]
+
+        # Re-order items
+        vzs = vzs[z_markers_idx]
+        vxs = vxs[z_markers_idx]
+        vys = vys[z_markers_idx]
+        if len(fcs) > 1:
+            fcs = fcs[z_markers_idx]
+        if len(ecs) > 1:
+            ecs = ecs[z_markers_idx]
+        if len(sizes) > 1:
+            sizes = sizes[z_markers_idx]
+        if len(lws) > 1:
+            lws = lws[z_markers_idx]
+        vps = np.column_stack((vxs, vys))
+
+        fcs = mcolors.to_rgba_array(fcs, self._alpha)
         ecs = mcolors.to_rgba_array(ecs, self._alpha)
+
         self.set_edgecolors(ecs)
-        PathCollection.set_offsets(self, np.column_stack([vxs, vys]))
+        self.set_facecolors(fcs)
+        self.set_sizes(sizes)
+        self.set_linewidth(lws)
+
+        PathCollection.set_offsets(self, vps)
 
         return np.min(vzs) if vzs.size else np.nan
 
@@ -580,17 +557,39 @@ def patch_collection_2d_to_3d(col, zs=0, zdir='z', depthshade=True):
 class Poly3DCollection(PolyCollection):
     """
     A collection of 3D polygons.
+
+    .. note::
+        **Filling of 3D polygons**
+
+        There is no simple definition of the enclosed surface of a 3D polygon
+        unless the polygon is planar.
+
+        In practice, Matplotlib fills the 2D projection of the polygon. This
+        gives a correct filling appearance only for planar polygons. For all
+        other polygons, you'll find orientations in which the edges of the
+        polygon intersect in the projection. This will lead to an incorrect
+        visualization of the 3D area.
+
+        If you need filled areas, it is recommended to create them via
+        `~mpl_toolkits.mplot3d.axes3d.Axes3D.plot_trisurf`, which creates a
+        triangulation and thus generates consistent surfaces.
     """
 
     def __init__(self, verts, *args, zsort='average', **kwargs):
         """
-        Create a Poly3DCollection.
-
-        *verts* should contain 3D coordinates.
-
-        Keyword arguments:
-        zsort, see set_zsort for options.
+        Parameters
+        ----------
+        verts : list of array-like Nx3
+            Each element describes a polygon as a sequence of ``N_i`` points
+            ``(x, y, z)``.
+        zsort : {'average', 'min', 'max'}, default: 'average'
+            The calculation method for the z-order.
+            See `~.Poly3DCollection.set_zsort` for details.
+        *args, **kwargs
+            All other parameters are forwarded to `.PolyCollection`.
 
+        Notes
+        -----
         Note that this class does a bit of magic with the _facecolors
         and _edgecolors properties.
         """
@@ -606,46 +605,29 @@ class Poly3DCollection(PolyCollection):
 
     def set_zsort(self, zsort):
         """
-        Sets the calculation method for the z-order.
+        Set the calculation method for the z-order.
 
         Parameters
         ----------
         zsort : {'average', 'min', 'max'}
             The function applied on the z-coordinates of the vertices in the
-            viewer's coordinate system, to determine the z-order.  *True* is
-            deprecated and equivalent to 'average'.
+            viewer's coordinate system, to determine the z-order.
         """
-        if zsort is True:
-            cbook.warn_deprecated(
-                "3.1", message="Passing True to mean 'average' for set_zsort "
-                "is deprecated and support will be removed in Matplotlib 3.3; "
-                "pass 'average' instead.")
-            zsort = 'average'
         self._zsortfunc = self._zsort_functions[zsort]
         self._sort_zpos = None
         self.stale = True
 
     def get_vector(self, segments3d):
         """Optimize points for projection."""
-        si = 0
-        ei = 0
-        segis = []
-        points = []
-        for p in segments3d:
-            points.extend(p)
-            ei = si + len(p)
-            segis.append((si, ei))
-            si = ei
-
         if len(segments3d):
-            xs, ys, zs = zip(*points)
-        else:
-            # We need this so that we can skip the bad unpacking from zip()
+            xs, ys, zs = np.row_stack(segments3d).T
+        else:  # row_stack can't stack zero arrays.
             xs, ys, zs = [], [], []
-
         ones = np.ones(len(xs))
         self._vec = np.array([xs, ys, zs, ones])
-        self._segis = segis
+
+        indices = [0, *np.cumsum([len(segment) for segment in segments3d])]
+        self._segslices = [*map(slice, indices[:-1], indices[1:])]
 
     def set_verts(self, verts, closed=True):
         """Set 3D vertices."""
@@ -655,7 +637,7 @@ class Poly3DCollection(PolyCollection):
         self._closed = closed
 
     def set_verts_and_codes(self, verts, codes):
-        """Sets 3D vertices with path codes."""
+        """Set 3D vertices with path codes."""
         # set vertices with closed=False to prevent PolyCollection from
         # setting path codes
         self.set_verts(verts, closed=False)
@@ -688,8 +670,7 @@ class Poly3DCollection(PolyCollection):
             self._facecolors3d = self._facecolors
 
         txs, tys, tzs = proj3d._proj_transform_vec(self._vec, renderer.M)
-        xyzlist = [(txs[si:ei], tys[si:ei], tzs[si:ei])
-                   for si, ei in self._segis]
+        xyzlist = [(txs[sl], tys[sl], tzs[sl]) for sl in self._segslices]
 
         # This extra fuss is to re-order face / edge colors
         cface = self._facecolors3d
@@ -709,17 +690,16 @@ class Poly3DCollection(PolyCollection):
              in enumerate(zip(xyzlist, cface, cedge))),
             key=lambda x: x[0], reverse=True)
 
-        segments_2d = [s for z, s, fc, ec, idx in z_segments_2d]
+        zzs, segments_2d, self._facecolors2d, self._edgecolors2d, idxs = \
+            zip(*z_segments_2d)
+
         if self._codes3d is not None:
-            codes = [self._codes3d[idx] for z, s, fc, ec, idx in z_segments_2d]
+            codes = [self._codes3d[idx] for idx in idxs]
             PolyCollection.set_verts_and_codes(self, segments_2d, codes)
         else:
             PolyCollection.set_verts(self, segments_2d, self._closed)
 
-        self._facecolors2d = [fc for z, s, fc, ec, idx in z_segments_2d]
-        if len(self._edgecolors3d) == len(cface):
-            self._edgecolors2d = [ec for z, s, fc, ec, idx in z_segments_2d]
-        else:
+        if len(self._edgecolors3d) != len(cface):
             self._edgecolors2d = self._edgecolors3d
 
         # Return zorder value
@@ -744,18 +724,7 @@ class Poly3DCollection(PolyCollection):
         self._edgecolors3d = PolyCollection.get_edgecolor(self)
 
     def set_alpha(self, alpha):
-        """
-        Set the alpha transparencies of the collection.
-
-        Parameters
-        ----------
-        alpha : float or None
-        """
-        if alpha is not None:
-            try:
-                float(alpha)
-            except TypeError:
-                raise TypeError('alpha must be a float or None')
+        # docstring inherited
         artist.Artist.set_alpha(self, alpha)
         try:
             self._facecolors3d = mcolors.to_rgba_array(
@@ -828,12 +797,6 @@ def _get_colors(c, num):
         (num, 4))
 
 
-@cbook.deprecated("3.1")
-def get_colors(c, num):
-    """Stretch the color argument to provide the required number *num*."""
-    return _get_colors(c, num)
-
-
 def _zalpha(colors, zs):
     """Modify the alphas of the color list according to depth."""
     # FIXME: This only works well if the points for *zs* are well-spaced
@@ -846,9 +809,3 @@ def _zalpha(colors, zs):
     sats = 1 - norm(zs) * 0.7
     rgba = np.broadcast_to(mcolors.to_rgba_array(colors), (len(zs), 4))
     return np.column_stack([rgba[:, :3], rgba[:, 3] * sats])
-
-
-@cbook.deprecated("3.1")
-def zalpha(colors, zs):
-    """Modify the alphas of the color list according to depth."""
-    return _zalpha(colors, zs)
